--[[
    功能：关节运动
    描述：以关节运动的方式依次经过3个路点
]]
local aubo = require('aubo')
local sched = sched or aubo.sched
local math = aubo.math or math

local sleep = sched.sleep
local thread = sched.thread
local sync = sched.sync
local run = sched.run
local kill = sched.kill
function p_example_movej()
    setDigitalInputActionDefault()
    setDigitalOutputRunstateDefault()
    setPayload(0, {0,0,0}, {0,0,0}, {0,0,0,0,0,0})
    setTcpOffset({0.0003,0.0001,0.00066,0,0,0})
    Base = {0,0,0,0,0,0}
    Tool = {0,0,0,0,0,0}
    
    function waitForMotionComplete()
        while (getQueueSize() > 0) or (isSteady() == false) do
            sync()
        end
    end
    
    local function calculate_point_to_move_towards(feature, direction, position_distance)
      local posDir={direction[1], direction[2], direction[3]}
      if (math.norm(posDir) < 1e-6) then
          return getTargetTcpPose()
      end
      local direction_vector_normalized=math.normalize(posDir)
      local displacement_pose={direction_vector_normalized[1] * position_distance,direction_vector_normalized[2] * position_distance,direction_vector_normalized[3] * position_distance,0,0,0}

      local wanted_displacement_in_base_frame=poseSub(poseTrans(feature, displacement_pose), feature)
      return poseAdd(getTargetTcpPose(), wanted_displacement_in_base_frame)
    end
    setPlanContext(0, 1, "Program")
    setPlanContext(0, 2, "example_movej.lua")
    pi = 3.14159265358979323846
    -- 关节角，单位：弧度
    q1 = {-0.93/180*pi, -15.77/180*pi, 97.57/180*pi, 23.33/180*pi, 89.99/180*pi, -1.82/180*pi}
    q2 = {-16.67/180*pi, -21.25/180*pi, 56.19/180*pi, -12.57/180*pi, 89.99/180*pi, -17.56/180*pi}
    q3 = {30.22/180*pi, -14.97/180*pi, 61.42/180*pi, -13.62/180*pi, 89.99/180*pi, 29.33/180*pi}
    -- 以关节运动的方式依次经过3个路点：q1、q2和q3
    moveJoint(q1, 1, 1, 0, 0)
    textmsg("joint move to waypoint1")
    moveJoint(q2, 1, 1, 0, 0)
    textmsg("joint move to waypoint2")
    moveJoint(q3, 1, 1, 0, 0)
    textmsg("joint move to waypoint3")
end

local plugin = {
  PRIORITY = 1000, -- set the plugin priority, which determines plugin execution order
  VERSION = "0.1",
  VENDOR = "Aubo Robotics",
}

function plugin:start_robot1()
  print("start_robot1---")
  -- 配置脚本环境
  local env = self.api:loadf(1)
  setmetatable(env, { __index = _G })
  env = require'aubo.scheduler'.trackGlobals(env) or env
  require('compat_env').setfenv(p_example_movej, env)
  p_example_movej()
end

function plugin:start(api)
  --
  self.api = api
  print("start---")
  self:start_robot1()
end

function plugin:robot_error_handler(name, err)
  --
  print("An error hanppen to robot "..name)
end

-- return our plugin object
return plugin
