--[[
    功能：直线运动
    步骤：
        第一步：关节运动到初始位置
        第二步：以直线运动的方式依次走3个路点 p1、p2和p3
]]
local aubo = require('aubo')
local sched = sched or aubo.sched
local math = aubo.math or math

local sleep = sched.sleep
local thread = sched.thread
local sync = sched.sync
local run = sched.run
local kill = sched.kill
function p_example_movel()
    setDigitalInputActionDefault()
    setDigitalOutputRunstateDefault()
    setPayload(0, {0,0,0}, {0,0,0}, {0,0,0,0,0,0})
    setTcpOffset({0.0003,0.0001,0.00066,0,0,0})
    Base = {0,0,0,0,0,0}
    Tool = {0,0,0,0,0,0}
    
    function waitForMotionComplete()
        while (getQueueSize() > 0) or (isSteady() == false) do
            sync()
        end
    end
    
    local function calculate_point_to_move_towards(feature, direction, position_distance)
      local posDir={direction[1], direction[2], direction[3]}
      if (math.norm(posDir) < 1e-6) then
          return getTargetTcpPose()
      end
      local direction_vector_normalized=math.normalize(posDir)
      local displacement_pose={direction_vector_normalized[1] * position_distance,direction_vector_normalized[2] * position_distance,direction_vector_normalized[3] * position_distance,0,0,0}

      local wanted_displacement_in_base_frame=poseSub(poseTrans(feature, displacement_pose), feature)
      return poseAdd(getTargetTcpPose(), wanted_displacement_in_base_frame)
    end
    setPlanContext(0, 1, "Program")
    setPlanContext(0, 2, "example_movel.lua")
    -- 关节运动到初始位置
    q = {-2.40194, 0.103747, 1.7804, 0.108636, 1.57129, -2.6379}
    moveJoint(q, 1, 1, 0, 0)
    textmsg("moving to initial position")
    -- 以直线运动的方式依次走3个路点： p1、p2和p3
    p1 = {-0.155944, -0.727344, 0.439066, 3.05165, 0.0324355, 1.80417}
    p2 = {-0.581143, -0.357548, 0.439066, 3.05165, 0.0324355, 1.80417}
    p3 = {0.503502, -0.420646, 0.439066, 3.05165, 0.0324355, 1.80417}
    moveLine(p1, 1, 1, 0, 0)
    textmsg("line move to waypoint1")
    moveLine(p2, 1, 1, 0, 0)
    textmsg("line move to waypoint2")
    moveLine(p3, 1, 1, 0, 0)
    textmsg("line move to waypoint3")
end

local plugin = {
  PRIORITY = 1000, -- set the plugin priority, which determines plugin execution order
  VERSION = "0.1",
  VENDOR = "Aubo Robotics",
}

function plugin:start_robot1()
  print("start_robot1---")
  -- 配置脚本环境
  local env = self.api:loadf(1)
  setmetatable(env, { __index = _G })
  env = require'aubo.scheduler'.trackGlobals(env) or env
  require('compat_env').setfenv(p_example_movel, env)
  p_example_movel()
end

function plugin:start(api)
  --
  self.api = api
  print("start---")
  self:start_robot1()
end

function plugin:robot_error_handler(name, err)
  --
  print("An error hanppen to robot "..name)
end

-- return our plugin object
return plugin