#include <cstring>
#include <fstream>
#include <math.h>

#include <nlohmann/json.hpp>

#include "aubo_sdk/rtde.h"
#include "aubo_sdk/rpc.h"
#include "aubo_sdk/script.h"

using namespace arcs::aubo_sdk;
using namespace arcs::common_interface;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

using namespace nlohmann;

void waitArrivel(RobotInterfacePtr impl)
{
    while (impl->getMotionControl()->getExecId() == -1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
    auto id = impl->getMotionControl()->getExecId();
    while (1) {
        auto id1 = impl->getMotionControl()->getExecId();
        if (id != id1) {
            break;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

template <typename T>
inline std::ostream &operator<<(std::ostream &os, const std::vector<T> &list)
{
    for (size_t i = 0; i < list.size(); i++) {
        os << list.at(i);
        if (i != (list.size() - 1)) {
            os << ",";
        }
    }
    return os;
}

#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc = std::make_shared<RpcClient>();
    rpc->connect(LOCAL_IP, 30004);
    rpc->login("aubo", "123456");

    std::ifstream f("../trajs/aubo-joint-test-0929-10.armplay");
    auto input = json::parse(f);
    std::vector<std::vector<double>> traj = input["jointlist"];
    double interval = input["interval"];

    auto traj_sz = traj.size();
    if (traj_sz == 0) {
        // 没有轨迹
        std::cerr << "No waypoints" << std::endl;
        return 0;
    } else {
        std::cout << "Load points size: " << traj.size() << std::endl;
    }
    std::vector<double> q1 = traj[0];

    // 移动到第一个点
    printf("goto p1\n");

    auto robot_name = rpc->getRobotNames().front();
    auto mc = rpc->getRobotInterface(robot_name)->getMotionControl();

    rpc->getRuntimeMachine()->start();
    mc->moveJoint(q1, M_PI, M_PI, 0., 0.);
    waitArrivel(rpc->getRobotInterface(robot_name));

    std::cout << "pathBufferAlloc " << std::endl;
    mc->pathBufferFree("rec");
    mc->pathBufferAlloc("rec", 2, traj_sz);

    size_t offset = 10;
    auto it = traj.begin();
    while (true) {
        std::cout << "pathBufferAppend " << offset << std::endl;
        mc->pathBufferAppend("rec",
                             std::vector<std::vector<double>>{ it, it + 10 });
        it += 10;
        if (offset + 10 >= traj_sz) {
            std::cout << "pathBufferAppend " << traj_sz << std::endl;
            mc->pathBufferAppend(
                "rec", std::vector<std::vector<double>>{ it, traj.end() });
            break;
        }

        offset += 10;
    }

    std::cout << "pathBufferEval "
              << mc->pathBufferEval("rec", {}, {}, interval) << std::endl;
    while (!mc->pathBufferValid("rec")) {
        std::cout << "pathBufferValid " << std::endl;
        std::this_thread::sleep_for(std::chrono::milliseconds(5));
    }
    mc->movePathBuffer("rec");
    waitArrivel(rpc->getRobotInterface(robot_name));

    rpc->disconnect();

    std::cout << "end servoj " << std::endl;
    return 0;
}
