#include "example_3.h"

#include <unistd.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <sstream>
#include <fstream>
#include "util.h"

#define SERVER_HOST "127.0.0.1"
#define SERVER_PORT 30000
void Example_3::demo()
{
    ServiceInterface robotservice;

    int ret = aubo_robot_namespace::InterfaceCallSuccCode;

    /** Interface call: login ***/
    ret = robotservice.robotServiceLogin(SERVER_HOST, SERVER_PORT, "aubo",
                                         "123456");
    if (ret == aubo_robot_namespace::InterfaceCallSuccCode) {
        std::cout << "login successful." << std::endl;
    } else {
        std::cerr << "login failed." << std::endl;
    }

    std::cout << "Robot arm initialization....." << std::endl;

    /** If the real robot arm is connected, the arm needs to be initialized.**/
    aubo_robot_namespace::ROBOT_SERVICE_STATE result;

    /** Business block **/
    /** Interface call: Initialize motion properties ***/
    robotservice.robotServiceInitGlobalMoveProfile();

    /** Interface call: Set the maximum acceleration of the articulated motion
     * ***/
    aubo_robot_namespace::JointVelcAccParam jointMaxAcc;
    jointMaxAcc.jointPara[0] = 50.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[1] = 50.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[2] = 50.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[3] = 50.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[4] = 50.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[5] =
        50.0 / 180.0 * M_PI; // The interface requires the unit to be radians
    robotservice.robotServiceSetGlobalMoveJointMaxAcc(jointMaxAcc);

    /** Interface call: set the maximum speed of articulated motion ***/
    aubo_robot_namespace::JointVelcAccParam jointMaxVelc;
    jointMaxVelc.jointPara[0] = 80.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[1] = 80.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[2] = 80.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[3] = 80.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[4] = 80.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[5] =
        80.0 / 180.0 * M_PI; // The interface requires the unit to be radians
    robotservice.robotServiceSetGlobalMoveJointMaxVelc(jointMaxVelc);

    /** Robot arm movement to zero posture **/
    double jointAngle[aubo_robot_namespace::ARM_DOF] = { 0 };
    Util::initJointAngleArray(jointAngle, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

    std::cout << "Calling the motion function" << std::endl;
    ret = robotservice.robotServiceJointMove(jointAngle, true);
    if (ret != aubo_robot_namespace::InterfaceCallSuccCode) {
        std::cerr << "Movement to zero posture failure.　ret:" << ret
                  << std::endl;
    }

    for (int i = 0; i > -1; i++) {
        //        /** Interface call: set offset**/
        //        aubo_robot_namespace::MoveRelative relativeMoveOnBase;
        //        relativeMoveOnBase.ena = true;
        //        relativeMoveOnBase.relativePosition[0] = 0;
        //        relativeMoveOnBase.relativePosition[1] = 0;
        //        relativeMoveOnBase.relativePosition[2] = 0.05*(i%4);   //Unit:
        //        m

        //        relativeMoveOnBase.relativeOri.w=1;
        //        relativeMoveOnBase.relativeOri.x=0;
        //        relativeMoveOnBase.relativeOri.y=0;
        //        relativeMoveOnBase.relativeOri.z=0;
        //        robotservice.robotServiceSetMoveRelativeParam(relativeMoveOnBase);
        std::cout << "Movement to waypoint 1" << std::endl;
        /** Waypoint 1 movement **/
        Util::initJointAngleArray(jointAngle, 0.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI, 0.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI, 0.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI);
        ret = robotservice.robotServiceJointMove(jointAngle, true);
        if (ret != aubo_robot_namespace::InterfaceCallSuccCode) {
            std::cerr << "Movement to waypoint 1 failed.　ret:" << ret
                      << std::endl;
            break;
        }
        std::cout << "Movement to waypoint 2" << std::endl;
        /** Waypoint 2 movement **/
        Util::initJointAngleArray(jointAngle, 90.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI, 0.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI, 0.0 / 180.0 * M_PI,
                                  0.0 / 180.0 * M_PI);
        ret = robotservice.robotServiceJointMove(jointAngle, true);
        if (ret != aubo_robot_namespace::InterfaceCallSuccCode) {
            std::cerr << "Movement to waypoint 2 failed.　ret:" << ret
                      << std::endl;
            break;
        }
        aubo_robot_namespace::ForceSensorData data;
        ret = robotservice.robotServerGetToolForceSensorData(data);
        double tmp[6];
        tmp[0] = data.data[0];
        tmp[1] = data.data[1];
        tmp[2] = data.data[2];
        tmp[3] = data.data[3];
        tmp[4] = data.data[4];
        tmp[5] = data.data[5];
        std::cout << "forcesenser data : " << ret << std::endl;

        // for(int j = 0; j < 6; j++) {
        //    std::cout<<"data"<<j<<":"<<data.data[j]<<std::endl;
        //}
        std::cout << "data" << 0 << ":" << tmp[0] << std::endl;
        std::cout << "data" << 1 << ":" << tmp[1] << std::endl;
        std::cout << "data" << 2 << ":" << tmp[2] << std::endl;
        std::cout << "data" << 3 << ":" << tmp[3] << std::endl;
        std::cout << "data" << 4 << ":" << tmp[4] << std::endl;
        std::cout << "data" << 5 << ":" << tmp[5] << std::endl;
    }

    sleep(10);

    /** Robotic arm shutdown **/
    robotservice.robotServiceRobotShutdown();

    /** Interface call: logout　**/
    robotservice.robotServiceLogout();
}
void Example_3::Relative_demo()
{
    ServiceInterface robotservice;

    int ret = aubo_robot_namespace::InterfaceCallSuccCode;

    /** Interface call: login ***/
    ret = robotservice.robotServiceLogin(SERVER_HOST, SERVER_PORT, "aubo",
                                         "123456");
    if (ret == aubo_robot_namespace::InterfaceCallSuccCode) {
        std::cout << "login successful." << std::endl;
    } else {
        std::cerr << "login failed." << std::endl;
    }

    std::cout << "Robot arm initialization....." << std::endl;

    /** Interface call: Initialize motion properties ***/
    robotservice.robotServiceInitGlobalMoveProfile();

    /** Interface call: Set the maximum acceleration of the articulated motion
     * ***/
    aubo_robot_namespace::JointVelcAccParam jointMaxAcc;
    jointMaxAcc.jointPara[0] = 10.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[1] = 10.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[2] = 10.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[3] = 10.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[4] = 10.0 / 180.0 * M_PI;
    jointMaxAcc.jointPara[5] =
        10.0 / 180.0 * M_PI; // The interface requires the unit to be radians
    robotservice.robotServiceSetGlobalMoveJointMaxAcc(jointMaxAcc);

    /** Interface call: set the maximum speed of articulated motion ***/
    aubo_robot_namespace::JointVelcAccParam jointMaxVelc;
    jointMaxVelc.jointPara[0] = 10.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[1] = 10.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[2] = 10.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[3] = 10.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[4] = 10.0 / 180.0 * M_PI;
    jointMaxVelc.jointPara[5] =
        10.0 / 180.0 * M_PI; // The interface requires the unit to be radians
    robotservice.robotServiceSetGlobalMoveJointMaxVelc(jointMaxVelc);

    /** Robot arm movement to zero posture **/
    double jointAngle[aubo_robot_namespace::ARM_DOF] = { 0 };

    Util::initJointAngleArray(jointAngle, 0.0, -15 / 180.0 * M_PI,
                              100 / 180.0 * M_PI, 25 / 180.0 * M_PI,
                              90 / 180.0 * M_PI, 0.0);

    std::cout << "Calling the motion function" << std::endl;
    ret = robotservice.robotServiceJointMove(jointAngle, true);
    if (ret != aubo_robot_namespace::InterfaceCallSuccCode) {
        std::cerr << "Movement to zero posture failure.　ret:" << ret
                  << std::endl;
    }

    /** Interface call: login ***/
    aubo_robot_namespace::MoveRelative relativeMoveOnBase;
    relativeMoveOnBase.ena = true;
    relativeMoveOnBase.relativePosition[0] = 0; // -0.04887;
    relativeMoveOnBase.relativePosition[1] = 0.0215;
    relativeMoveOnBase.relativePosition[2] = 0; // Unit:m

    relativeMoveOnBase.relativeOri.w = 1;
    relativeMoveOnBase.relativeOri.x = 0;
    relativeMoveOnBase.relativeOri.y = 0;
    relativeMoveOnBase.relativeOri.z = 0;
    //    relativeMoveOnBase.relativeOri.w = 1;
    //    relativeMoveOnBase.relativeOri.x = 0;
    //    relativeMoveOnBase.relativeOri.y = 0;
    //    relativeMoveOnBase.relativeOri.z = 0;
    //    ret =
    //    robotservice.robotServiceSetMoveRelativeParam(relativeMoveOnBase);

    aubo_robot_namespace::MoveProfile_t moveProfile;
    moveProfile.relative = relativeMoveOnBase;
    memcpy(moveProfile.jointMaxAcc, jointMaxAcc.jointPara,
           sizeof(jointMaxAcc.jointPara));
    memcpy(moveProfile.jointMaxVelc, jointMaxVelc.jointPara,
           sizeof(jointMaxAcc.jointPara));

    aubo_robot_namespace::CoordCalibrateByJointAngleAndTool
        userCoord;                                     //用户坐标系
    aubo_robot_namespace::ToolInEndDesc toolInEndDesc; //工具参数
    toolInEndDesc.toolInEndPosition.x = 0;
    toolInEndDesc.toolInEndPosition.y = 0;
    toolInEndDesc.toolInEndPosition.z = 0;
    toolInEndDesc.toolInEndOrientation.w = 1.0;
    toolInEndDesc.toolInEndOrientation.x = 0;
    toolInEndDesc.toolInEndOrientation.y = 0;
    toolInEndDesc.toolInEndOrientation.z = 0;

    userCoord.coordType = aubo_robot_namespace::WorldCoordinate;
    userCoord.methods = aubo_robot_namespace::
        Origin_AnyPointOnPositiveXAxis_AnyPointOnFirstQuadrantOfXOYPlane;
    double radian_tem[6] = { 0.0,
                             -15 / 180.0 * M_PI,
                             100 / 180.0 * M_PI,
                             25 / 180.0 * M_PI,
                             90 / 180.0 * M_PI,
                             0.0 };
    memcpy(userCoord.wayPointArray[0].jointPos, radian_tem, sizeof(radian_tem));

    double radian_tem2[6] = { 9.578 / 180.0 * M_PI,   -13.35 / 180.0 * M_PI,
                              102.133 / 180.0 * M_PI, 25.484 / 180.0 * M_PI,
                              90 / 180.0 * M_PI,      9.578 / 180.0 * M_PI };
    memcpy(userCoord.wayPointArray[1].jointPos, radian_tem2,
           sizeof(radian_tem2));

    double radian_tem3[6] = { 0,
                              11.472 / 180.0 * M_PI,
                              127.443 / 180.0 * M_PI,
                              25.970 / 180.0 * M_PI,
                              90 / 180.0 * M_PI,
                              0 };
    memcpy(userCoord.wayPointArray[2].jointPos, radian_tem3,
           sizeof(radian_tem3));
    userCoord.toolDesc = toolInEndDesc;
    moveProfile.relativeOnCoord = userCoord;

    std::cout << "RelativeParam" << std::endl;
    ret = robotservice.robotServiceJointMove(moveProfile, jointAngle, true);
    if (ret != aubo_robot_namespace::InterfaceCallSuccCode) {
        std::cerr << "Movement to zero posture failure.　ret:" << ret
                  << std::endl;
    }
}
