#include "aubo_sdk/rpc.h"
#include <unordered_map>

using namespace arcs::aubo_sdk;

// 模板: 打印数据类型为std::vector<T>的函数返回值
template <typename T>
void printVec(std::vector<T> param, std::string name)
{
    // std::cout << "@:" << name << std::endl;
    for (int i = 0; i < param.size(); i++) {
        std::cout << param.at(i);
        if (i != param.size() - 1) {
            std::cout << ", ";
        }
    }
    std::cout << std::endl;
}

void exampleConfig(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();

    // 接口调用: 获取机器人的名字
    std::string name;
    name = cli->getRobotInterface(robot_name)->getRobotConfig()->getName();
    std::cout << "机器人的名字：" << name << std::endl;

    // 接口调用: 获取机器人的自由度
    int dof;
    dof = cli->getRobotInterface(robot_name)->getRobotConfig()->getDof();
    std::cout << "机器人的自由度：" << dof << std::endl;

    // 接口调用: 获取机器人的伺服控制周期(从硬件抽象层读取)
    double cycle_time;
    cycle_time =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getCycletime();
    std::cout << "伺服控制周期：" << cycle_time << std::endl;

    // 接口调用: 获取默认的工具端加速度，单位m/s^2
    double default_tool_acc;
    default_tool_acc = cli->getRobotInterface(robot_name)
                           ->getRobotConfig()
                           ->getDefaultToolAcc();
    std::cout << "默认的工具端加速度： " << default_tool_acc << std::endl;

    // 接口调用: 获取默认的工具端速度，单位m/s
    double default_tool_speed;
    default_tool_speed = cli->getRobotInterface(robot_name)
                             ->getRobotConfig()
                             ->getDefaultToolSpeed();
    std::cout << "默认的工具端速度：" << default_tool_speed << std::endl;

    // 接口调用: 获取默认的关节加速度，单位rad/s^2
    double default_joint_acc;
    default_joint_acc = cli->getRobotInterface(robot_name)
                            ->getRobotConfig()
                            ->getDefaultJointAcc();
    std::cout << "默认的关节加速度：" << default_joint_acc << std::endl;

    // 接口调用: 获取默认的关节速度，单位rad/s
    double default_joint_speed;
    default_joint_speed = cli->getRobotInterface(robot_name)
                              ->getRobotConfig()
                              ->getDefaultJointSpeed();
    std::cout << "默认的关节速度：" << default_joint_speed << std::endl;

    // 接口调用: 获取机器人类型代码
    std::string robot_type;
    robot_type =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getRobotType();
    std::cout << "机器人类型代码：" << robot_type << std::endl;

    // 接口调用: 获取机器人子类型代码
    std::string sub_type;
    sub_type =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getRobotSubType();
    std::cout << "机器人子类型代码：" << sub_type << std::endl;

    // 接口调用: 获取控制柜类型代码
    int control_box_type;
    control_box_type = cli->getRobotInterface(robot_name)
                           ->getRobotConfig()
                           ->getControlBoxType();
    std::cout << "控制柜类型代码：" << control_box_type << std::endl;

    // 接口调用: 获取安装位姿(机器人的基坐标系相对于世界坐标系)
    std::vector<double> mounting_pose;
    mounting_pose =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getMountingPose();
    std::cout << "机器人的安装位姿：";
    printVec<double>(mounting_pose, "mounting_pose");

    // 接口调用: 设置碰撞灵敏度等级
    int level = 6;
    int ret1 = cli->getRobotInterface(robot_name)
                   ->getRobotConfig()
                   ->setCollisionLevel(level);
    std::cout << "设置碰撞灵敏度等级：" << level << std::endl;
    std::cout << "ret1: " << ret1 << std::endl;

    // 接口调用: 获取碰撞灵敏度等级
    int collision_level;
    collision_level = cli->getRobotInterface(robot_name)
                          ->getRobotConfig()
                          ->getCollisionLevel();
    std::cout << "碰撞灵敏度等级：" << collision_level << std::endl;

    // 接口调用: 设置碰撞停止类型
    int type = 1;
    int ret2 = cli->getRobotInterface(robot_name)
                   ->getRobotConfig()
                   ->setCollisionStopType(type);
    std::cout << "设置碰撞停止类型：" << type << std::endl;
    std::cout << "ret2 = " << ret2 << std::endl;

    // 接口调用: 获取碰撞停止类型
    int collision_stop_type;
    collision_stop_type = cli->getRobotInterface(robot_name)
                              ->getRobotConfig()
                              ->getCollisionStopType();
    std::cout << "碰撞停止类型：" << collision_stop_type << std::endl;

    // 接口调用: 设置工具电压输出
    int domain = 5;
    cli->getRobotInterface(robot_name)
        ->getRobotConfig()
        ->setToolVoltageOutputDomain(domain);
    std::cout << "设置工具电压输出： " << domain << std::endl;

    // 接口调用: 获取工具电压输出
    int output_domain;
    output_domain = cli->getRobotInterface(robot_name)
                        ->getRobotConfig()
                        ->getToolVoltageOutputDomain();
    std::cout << "工具电压输出： " << output_domain << std::endl;

    // 接口调用: 获取机器人DH参数
    std::unordered_map<std::string, std::vector<double>> kin_param;
    kin_param = cli->getRobotInterface(robot_name)
                    ->getRobotConfig()
                    ->getKinematicsParam(true);
    std::cout << "机器人DH参数：" << std::endl;
    for (auto it = kin_param.begin(); it != kin_param.end(); it++) {
        std::cout << it->first << ": ";
        int size = it->second.size();
        for (int i = 0; i < size; i++) {
            std::cout << it->second.at(i) << " ";
        }
        std::cout << std::endl;
    }

    // 接口调用: 获取指定温度下的DH参数补偿值
    std::unordered_map<std::string, std::vector<double>> kin_compensate;
    double temperature = 20;
    kin_compensate = cli->getRobotInterface(robot_name)
                         ->getRobotConfig()
                         ->getKinematicsCompensate(temperature);
    std::cout << "机器人DH参数补偿值：" << std::endl;
    for (auto it = kin_compensate.begin(); it != kin_compensate.end(); it++) {
        std::cout << it->first << ": ";
        int size = it->second.size();
        for (int i = 0; i < size; i++) {
            std::cout << it->second.at(i) << " ";
        }
        std::cout << std::endl;
    }

    // 接口调用: 获取可用的末端力矩传感器的名字
    std::vector<std::string> tcp_force_sensor_names;
    tcp_force_sensor_names = cli->getRobotInterface(robot_name)
                                 ->getRobotConfig()
                                 ->getTcpForceSensorNames();
    std::cout << "末端力矩传感器的名字：";
    printVec<std::string>(tcp_force_sensor_names, "tcp_force_sensor_names");

    // 接口调用: 获取末端力矩偏移
    std::vector<double> tcp_force_offset;
    tcp_force_offset = cli->getRobotInterface(robot_name)
                           ->getRobotConfig()
                           ->getTcpForceOffset();
    std::cout << "末端力矩偏移：";
    printVec<double>(tcp_force_offset, "tcp_force_offset");

    // 接口调用: 获取可用的底座力矩传感器的名字
    std::vector<std::string> base_force_sensor_names;
    base_force_sensor_names = cli->getRobotInterface(robot_name)
                                  ->getRobotConfig()
                                  ->getBaseForceSensorNames();
    std::cout << "可用的底座力矩传感器的名字：";
    printVec<std::string>(base_force_sensor_names, "base_force_sensor_names");

    // 接口调用: 获取底座力矩偏移
    std::vector<double> base_force_offset;
    base_force_offset = cli->getRobotInterface(robot_name)
                            ->getRobotConfig()
                            ->getBaseForceOffset();
    std::cout << "底座力矩偏移：";
    printVec<double>(base_force_offset, "base_force_offset");

    // 接口调用: 获取安全参数校验码 CRC32
    uint32_t check_sum;
    check_sum = cli->getRobotInterface(robot_name)
                    ->getRobotConfig()
                    ->getSafetyParametersCheckSum();
    std::cout << "获取安全参数校验码 CRC32：" << check_sum << std::endl;

    // 接口调用: 获取关节最大位置（物理极限）
    std::vector<double> joint_max_positions;
    joint_max_positions = cli->getRobotInterface(robot_name)
                              ->getRobotConfig()
                              ->getJointMaxPositions();
    std::cout << "获取关节最大位置（物理极限）：";
    printVec<double>(joint_max_positions, "joint_max_positions");

    // 接口调用: 获取关节最小位置（物理极限）
    std::vector<double> joint_min_positions;
    joint_min_positions = cli->getRobotInterface(robot_name)
                              ->getRobotConfig()
                              ->getJointMinPositions();
    std::cout << "获取关节最小位置（物理极限）：";
    printVec<double>(joint_min_positions, "joint_min_positions");

    // 接口调用: 获取关节最大速度（物理极限）
    std::vector<double> joint_max_speeds;
    joint_max_speeds = cli->getRobotInterface(robot_name)
                           ->getRobotConfig()
                           ->getJointMaxSpeeds();
    std::cout << "获取关节最大速度（物理极限）：";
    printVec<double>(joint_max_speeds, "joint_max_speeds");

    // 接口调用: 获取关节最大加速度（物理极限）
    std::vector<double> joint_max_acc;
    joint_max_acc = cli->getRobotInterface(robot_name)
                        ->getRobotConfig()
                        ->getJointMaxAccelerations();
    std::cout << "获取关节最大加速度（物理极限）：";
    printVec<double>(joint_max_acc, "joint_max_acc");

    // 接口调用: 获取TCP最大速度（物理极限）
    std::vector<double> tcp_max_speeds;
    tcp_max_speeds =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getTcpMaxSpeeds();
    std::cout << "获取TCP最大速度（物理极限）：";
    printVec<double>(tcp_max_speeds, "tcp_max_speeds");

    // 接口调用: 获取TCP最大加速度（物理极限）
    std::vector<double> tcp_max_acc;
    tcp_max_acc = cli->getRobotInterface(robot_name)
                      ->getRobotConfig()
                      ->getTcpMaxAccelerations();
    std::cout << "获取TCP最大加速度（物理极限）：";
    printVec<double>(tcp_max_acc, "tcp_max_acc");

    // 接口调用:获取机器人的安装姿态
    std::vector<double> gravity;
    gravity =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getGravity();
    std::cout << "获取机器人的安装姿态：";
    printVec<double>(gravity, "gravity");

    // 接口调用: 获取TCP偏移
    std::vector<double> tcp_offset;
    tcp_offset =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getTcpOffset();
    std::cout << "获取TCP偏移：";
    printVec<double>(tcp_offset, "tcp_offset");

    // 接口调用: 获取末端负载
    Payload payload;
    payload =
        cli->getRobotInterface(robot_name)->getRobotConfig()->getPayload();
    std::cout << "末端负载：" << std::endl;
    std::cout << "mass：" << std::get<0>(payload) << std::endl;
    std::vector<double> cog = std::get<1>(payload);
    std::cout << "cog：";
    for (int i = 0; i < cog.size(); i++) {
        std::cout << cog.at(i) << " ";
        if (i == cog.size() - 1)
            std::cout << std::endl;
    }
    std::vector<double> aom = std::get<2>(payload);
    std::cout << "aom：";
    for (int i = 0; i < aom.size(); i++) {
        std::cout << aom.at(i) << " ";
        if (i == aom.size() - 1)
            std::cout << std::endl;
    }
    std::vector<double> intertia = std::get<3>(payload);
    std::cout << "intertia：";
    for (int i = 0; i < intertia.size(); i++) {
        std::cout << intertia.at(i) << " ";
        if (i == intertia.size() - 1)
            std::cout << std::endl;
    }

    // 接口调用: 获取固件升级的进程
    std::tuple<std::string, double> firmware_update_process;
    firmware_update_process = cli->getRobotInterface(robot_name)
                                  ->getRobotConfig()
                                  ->getFirmwareUpdateProcess();
    std::cout << "获取固件升级的进程：";
    std::cout << "步骤名称：" << std::get<0>(firmware_update_process)
              << "进度：" << std::get<1>(firmware_update_process) << std::endl;
}

#define ip_local "127.0.0.1"

/**
 * 功能: 获取机械臂配置信息
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 获取机械臂相关的配置信息
 */
int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(ip_local, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");

    //获取机械臂相关的配置信息
    exampleConfig(rpc_cli);
}
