#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

void exampleIo(RpcClientPtr impl)
{
    // 接口调用: 获取机器人的名字
    auto robot_name = impl->getRobotNames().front();
    auto robot = impl->getRobotInterface(robot_name);

    // 接口调用: 获取标准模拟输入数量
    int default_ai_num = robot->getIoControl()->getStandardAnalogInputNum();
    // 接口调用: 获取标准模拟输出数量
    int default_ao_num = robot->getIoControl()->getStandardAnalogOutputNum();
    // 接口调用: 获取标准数字输出数量
    int default_do_num = robot->getIoControl()->getStandardDigitalOutputNum();
    // 接口调用: 获取标准数字输入数量
    int default_di_num = robot->getIoControl()->getStandardDigitalInputNum();

    std::cout << "StandardDigitalInputNum: "
              << robot->getIoControl()->getStandardDigitalInputNum()
              << std::endl;
    std::cout << "StandardDigitalOutputNum: "
              << robot->getIoControl()->getStandardDigitalOutputNum()
              << std::endl;
    std::cout << "ToolDigitalInputNum: "
              << robot->getIoControl()->getToolDigitalInputNum() << std::endl;
    std::cout << "ToolDigitalOutputNum: "
              << robot->getIoControl()->getToolDigitalOutputNum() << std::endl;
    std::cout << "ConfigurableDigitalInputNum: "
              << robot->getIoControl()->getConfigurableDigitalInputNum()
              << std::endl;
    std::cout << "ConfigurableDigitalOutputNum: "
              << robot->getIoControl()->getConfigurableDigitalOutputNum()
              << std::endl;

    int ret = 0;
    // 接口调用: 设置标准数字输入触发动作为 Default
    ret = robot->getIoControl()->setStandardDigitalInputAction(
        0x00000001, StandardInputAction::Default);
    // 接口调用: 设置标准数字输出状态选择为 NONE
    ret = robot->getIoControl()->setStandardDigitalOutputRunstate(
        0x00000001, StandardOutputRunState::NONE);
    /*ret = impl->getIoControl()->setConfigurableInputAction(
        0x00000100, StandardInputAction::Freedrive);
    ret = impl->getIoControl()->setConfigurableDigitalOutputRunstate(
        0x00000100, StandardOutputRunState::STOP_HIGH);*/
    // 接口调用: 设置标准模拟输出状态选择为 RUNNING
    ret = robot->getIoControl()->setStandardAnalogOutputRunstate(
        0x00000010, StandardOutputRunState::RUNNING);
    // 接口调用: 设置标准模拟输入范围
    ret = robot->getIoControl()->setStandardAnalogInputDomain(0, 12);
    // 接口调用: 设置标准模拟输出范围
    ret = robot->getIoControl()->setStandardAnalogOutputDomain(0, 12);

    // configure io
    // [已测试] Digital IO [I系列IO控制响应频率:标准   20hz(max)]
    for (int i = 0; i < default_do_num; i++) {
        // 接口调用: 设置标准数字输出值
        ret = robot->getIoControl()->setStandardDigitalOutput(i, false);
        std::this_thread::sleep_for(std::chrono::seconds(1));
        // 接口调用: 获取标准数字输出值
        auto ret_b = robot->getIoControl()->getStandardDigitalOutput(i);
        std::cout << "Digital output:" << ret_b << std::endl;
    }

    for (int i = 0; i < default_di_num; i++) {
        // 接口调用: 获取标准数字输入值
        auto ret_b = robot->getIoControl()->getStandardDigitalInput(i);
        std::cout << "Digital input:" << ret_b << std::endl;
    }

    // Analog IO (四路模拟电压输入，两路模拟电压电流输出)
    // 接口调用: 设置标准模拟输出值
    ret = robot->getIoControl()->setStandardAnalogOutput(0x00000000,
                                                         5); // 0,1 Voltage V
    std::this_thread::sleep_for(std::chrono::seconds(1));
    // 接口调用: 设置标准模拟输出值
    ret = robot->getIoControl()->setStandardAnalogOutput(1, 10);
    std::this_thread::sleep_for(std::chrono::seconds(1));
    // 接口调用: 设置标准模拟输出值
    ret = robot->getIoControl()->setStandardAnalogOutput(2,
                                                         10); // 2,3 Current ma
    std::this_thread::sleep_for(std::chrono::seconds(1));
    // 接口调用: 设置标准模拟输出值
    ret = robot->getIoControl()->setStandardAnalogOutput(3, 10);
    std::this_thread::sleep_for(std::chrono::seconds(1));
    double ret_d = 0.0;
    for (int i = 0; i < default_ai_num; i++) {
        // 接口调用: 获取标准模拟输入值
        ret_d = robot->getIoControl()->getStandardAnalogInput(i);
        std::this_thread::sleep_for(std::chrono::seconds(1));
        std::cout << "Analog input Voltage:" << ret_d << std::endl;
    }
}

/**
 * 功能: IO
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 设置与获取机器人IO相关的参数
 */
#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(LOCAL_IP, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");
    // 设置与获取机器人IO相关的参数
    exampleIo(rpc_cli);
}
