#include "aubo_sdk/rpc.h"

using namespace arcs::aubo_sdk;

// 标准数字IO
void exampleStandardDigitalIO(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();
    auto robot = cli->getRobotInterface(robot_name);

    // 接口调用: 获取标准数字输入和输出数量
    std::cout << "StandardDigitalInputNum（标准数字输入数量）: "
              << robot->getIoControl()->getStandardDigitalInputNum()
              << std::endl;
    std::cout << "StandardDigitalOutputNum（标准数字输出数量）: "
              << robot->getIoControl()->getStandardDigitalOutputNum()
              << std::endl;

    // 接口调用: 设置所有的输入出发动作为Default
    robot->getIoControl()->setDigitalInputActionDefault();
    // 接口调用: 获取输入触发动作
    StandardInputAction input_action;
    input_action =
        robot->getIoControl()->getStandardDigitalInputAction(0x00000001);
    std::cout << "获取标准数字输入触发动作为：" << input_action << std::endl;
    // 接口调用: 设置输入出发动作为Freedrive
    input_action = StandardInputAction::Freedrive;
    robot->getIoControl()->setStandardDigitalInputAction(0x00000001,
                                                         input_action);
    std::cout << "设置标准数字输入触发动作为：" << input_action << std::endl;
    // 接口调用: 获取输入触发动作
    input_action =
        robot->getIoControl()->getStandardDigitalInputAction(0x00000001);
    std::cout << "获取标准数字输入触发动作为：" << input_action << std::endl;

    // 接口调用: 设置所有的输出状态选择为NONE
    robot->getIoControl()->setDigitalOutputRunstateDefault();
    // 接口调用: 获取输出状态选择
    StandardOutputRunState output_runstate;
    output_runstate =
        robot->getIoControl()->getStandardDigitalOutputRunstate(0x00000001);
    std::cout << "获取标准数字输出状态选择为：" << output_runstate << std::endl;
    // 接口调用: 设置输出状态选择为Running
    output_runstate = StandardOutputRunState::RUNNING;
    robot->getIoControl()->setStandardDigitalOutputRunstate(0x00000001,
                                                            output_runstate);
    std::cout << "设置标准数字输出状态选择为：" << output_runstate << std::endl;
    // 接口调用: 获取输出状态选择
    output_runstate =
        robot->getIoControl()->getStandardDigitalOutputRunstate(0x00000001);
    std::cout << "获取标准数字输出状态选择为：" << output_runstate << std::endl;

    // 接口调用: 获取标准数字输入和输出数量
    int default_do_num = robot->getIoControl()->getStandardDigitalOutputNum();
    int default_di_num = robot->getIoControl()->getStandardDigitalInputNum();
    // 打印所有的标准数字输出值
    for (int i = 0; i < default_do_num; i++) {
        robot->getIoControl()->setStandardDigitalOutput(i, false);
        std::this_thread::sleep_for(std::chrono::seconds(1));
        // 接口调用: 获取标准数字输出值
        auto ret_b = robot->getIoControl()->getStandardDigitalOutput(i);
        std::cout << "数字输出:" << ret_b << std::endl;
    }
    // 打印所有的标准数字输入值
    for (int i = 0; i < default_di_num; i++) {
        // 接口调用: 获取标准数字输入值
        auto ret_b = robot->getIoControl()->getStandardDigitalInput(i);
        std::cout << "数字输入:" << ret_b << std::endl;
    }
}

// 标准模拟IO
void exampleStandardAnalogIO(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();
    auto robot = cli->getRobotInterface(robot_name);

    // 接口调用: 获取标准模拟输入和输出数量
    std::cout << "StandardAnalogInputNum（标准模拟输入数量）: "
              << robot->getIoControl()->getStandardAnalogInputNum()
              << std::endl;
    std::cout << "StandardAnalogOutputNum（标准模拟输出数量）: "
              << robot->getIoControl()->getStandardAnalogOutputNum()
              << std::endl;

    // 接口调用: 设置和获取标准模拟输入范围
    int input_domain = 10;
    robot->getIoControl()->setStandardAnalogInputDomain(0, input_domain);
    std::cout << "设置标准模拟输入范围是:" << input_domain << std::endl;
    input_domain = robot->getIoControl()->getStandardAnalogInputDomain(0);
    std::cout << "获取标准模拟输入范围是:" << input_domain << std::endl;

    // 接口调用: 获取标准模拟输入
    double input_value = robot->getIoControl()->getStandardAnalogInput(0);
    std::cout << "获取标准模拟输入:" << input_value << std::endl;

    // 接口调用: 设置和获取标准模拟输出范围
    int output_domain = 10;
    robot->getIoControl()->setStandardAnalogOutputDomain(0, output_domain);
    std::cout << "设置标准模拟输出范围是:" << output_domain << std::endl;
    output_domain = robot->getIoControl()->getStandardAnalogOutputDomain(0);
    std::cout << "获取标准模拟输出范围是:" << output_domain << std::endl;

    // 接口调用: 设置和获取标准模拟输出状态选择
    StandardOutputRunState output_runstate = StandardOutputRunState::STOP_LOW;
    robot->getIoControl()->setStandardAnalogOutputRunstate(3, output_runstate);
    std::cout << "设置标准模拟输出状态选择:" << output_runstate << std::endl;
    output_runstate = robot->getIoControl()->getStandardAnalogOutputRunstate(3);
    std::cout << "获取标准模拟输出状态选择:" << output_runstate << std::endl;

    // 接口调用: 设置和获取标准模拟输出
    double output_value = 3.3;
    robot->getIoControl()->setStandardAnalogOutput(3, output_value);
    std::cout << "设置标准模拟输出:" << output_value << std::endl;
    output_value = robot->getIoControl()->getStandardAnalogOutput(3);
    std::cout << "获取标准模拟输出:" << output_value << std::endl;
}

// 工具端数字IO
void exampleToolDigitalIO(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();
    auto robot = cli->getRobotInterface(robot_name);

    // 接口调用: 获取工具端数字输入和输出数量
    std::cout << "ToolDigitalInputNum（工具端数字输入数量）: "
              << robot->getIoControl()->getToolDigitalInputNum() << std::endl;
    std::cout << "ToolDigitalOutputNum（工具端数字输出数量）: "
              << robot->getIoControl()->getToolDigitalOutputNum() << std::endl;

    // 接口调用: 设置指定的工具端IO为输入
    bool isInput = true;
    robot->getIoControl()->setToolIoInput(1, isInput);
    std::cout << "设置工具端IO:" << isInput << std::endl;
    // 接口调用: 判断指定的工具端IO是否为输入
    isInput = robot->getIoControl()->isToolIoInput(1);
    std::cout << "指定的工具端IO是否为输入:" << isInput << std::endl;
    // 接口调用: 获取工具端数字输入
    bool input = robot->getIoControl()->getToolDigitalInput(1);
    std::cout << "获取工具端数字输入:" << input << std::endl;

    // 接口调用: 设置和获取工具端数字输入触发动作
    StandardInputAction input_action = StandardInputAction::Default;
    robot->getIoControl()->setToolDigitalInputAction(0, input_action);
    std::cout << "设置工具端数字输入触发动作:" << input_action << std::endl;
    input_action = robot->getIoControl()->getToolDigitalInputAction(0);
    std::cout << "获取工具端数字输入触发动作:" << input_action << std::endl;

    // 接口调用: 设置和获取工具端数字输出状态选择
    StandardOutputRunState output_runstate = StandardOutputRunState::STOP_LOW;
    robot->getIoControl()->setToolDigitalOutputRunstate(3, output_runstate);
    std::cout << "设置工具端数字输出状态选择:" << output_runstate << std::endl;
    output_runstate = robot->getIoControl()->getToolDigitalOutputRunstate(3);
    std::cout << "获取工具端数字输出状态选择:" << output_runstate << std::endl;

    // 接口调用: 获取工具端数字输出
    bool output = true;
    robot->getIoControl()->setToolDigitalOutput(3, output);
    std::cout << "设置工具端数字输出:" << output << std::endl;
    output = robot->getIoControl()->getToolDigitalOutput(3);
    std::cout << "获取工具端数字输出:" << output << std::endl;
}

// 工具端模拟IO
void exampleToolAnalogIO(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();
    auto robot = cli->getRobotInterface(robot_name);

    // 接口调用: 获取工具端模拟输入和输出数量
    std::cout << "ToolAnalogInputNum（工具端模拟输入数量）: "
              << robot->getIoControl()->getToolAnalogInputNum() << std::endl;
    std::cout << "ToolAnalogOutputNum（工具端模拟输出数量）: "
              << robot->getIoControl()->getToolAnalogOutputNum() << std::endl;

    // 接口调用: 获取工具端模拟输入范围
    int input_domain = robot->getIoControl()->getToolAnalogInputDomain(1);
    std::cout << "获取工具端模拟输入范围:" << input_domain << std::endl;

    // 接口调用: 获取工具端模拟输入
    double input = robot->getIoControl()->getToolAnalogInput(1);
    std::cout << "获取工具端模拟输入:" << input << std::endl;
}

#define ip_local "127.0.0.1"

/**
 * 功能:IO
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 标准数字IO:
 *       1.获取标准数字输入和输出数量
 *       2.设置和获取输入触发动作
 *       3.设置和获取输出状态选择
 *       4.获取标准数字输入和输出值
 * 第三步: 标准模拟IO:
 *       1.获取标准模拟输入和输出数量
 *       2.设置和获取标准模拟输入范围
 *       3.获取标准模拟输入
 *       4.设置和获取标准模拟输出范围
 *       5.设置和获取标准模拟输出状态选择
 *       6.设置和获取标准模拟输出
 * 第四步: 工具端数字IO
 *       1.获取工具端数字输入和输出数量
 *       2.设置和获取工具端数字输入触发动作
 *       3.设置和获取工具端数字输出状态选择
 * 第五步: 工具端模拟IO
 *       1.获取工具端模拟输入和输出数量
 *       2.获取工具端模拟输入范围
 *       3.获取工具端模拟输入
 */
int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(ip_local, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");

    // 标准数字IO
    exampleStandardDigitalIO(rpc_cli);

    // 标准模拟IO
    exampleStandardAnalogIO(rpc_cli);

    // 工具端数字IO
    exampleToolDigitalIO(rpc_cli);

    // 工具端模拟IO
    exampleToolAnalogIO(rpc_cli);
}
