#include "aubo_sdk/rpc.h"
#include "aubo_sdk/rtde.h"
#include <aubo/error_stack/error_stack.h>
using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// 实现阻塞功能: 当机械臂运动到目标路点时，程序再往下执行
void waitArrivel(RobotInterfacePtr impl)
{
    while (impl->getMotionControl()->getExecId() == -1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
    auto id = impl->getMotionControl()->getExecId();
    while (1) {
        auto id1 = impl->getMotionControl()->getExecId();
        if (id != id1) {
            break;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

// 关节运动
void exampleMovej(RpcClientPtr cli)

{
    // 关节角，单位: 弧度
    std::vector<double> q1 = { -2.05177,  -0.400292, 1.19625,
                               0.0285152, 1.57033,   -2.28774 };

    std::vector<double> q2 = { -0.236948,   -0.465633, 1.1002,
                               -0.00600456, 1.56824,   -0.472864 };

    std::vector<double> q3 = { 1.38167,  -0.0115091, 1.66958,
                               0.107726, 1.572,      1.14576 };

    // 关节运动到路点 q1
    printf("goto p1\n");

    // 接口调用: 获取机器人的名字
    auto robot_name = cli->getRobotNames().front();

    // 接口调用: 关节运动
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0);
    // 阻塞
    waitArrivel(cli->getRobotInterface(robot_name));

    cli->getRobotInterface(robot_name)->getMotionControl()->pathOffsetEnable();
    std::vector<double> relativeonbasecoord(6);
    for (int i = 0; i < 6; i++) {
        relativeonbasecoord[i] = 0;
    }
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->pathOffsetSet(relativeonbasecoord);
    // 关节运动到路点 q2
    printf("goto p2\n");
    // 接口调用: 关节运动
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q2, 90 * (M_PI / 180), 180 * (M_PI / 180), 0, 0);
    // 阻塞
    waitArrivel(cli->getRobotInterface(robot_name));

    // 关节运动到路点 q3
    printf("goto p3\n");
    // 接口调用: 关节运动
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q3, 90 * (M_PI / 180), 180 * (M_PI / 180), 0, 0);
    // 阻塞
    waitArrivel(cli->getRobotInterface(robot_name));
}

void printlog(int level, const char *source, int code)
{
    static const char *level_names[] = { "Critical", "Error", "Warning",
                                         "Info",     "Debug", "BackTrace" };
    fprintf(stderr, "[%s] %s - %d %s\n", level_names[level], source, code,
            arcs::error_stack::errorCode2Str(code));
}

/**
 * 功能: 机械臂关节运动
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 以关节运动的方式依次经过3个路点
 */
#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    cli->connect(LOCAL_IP, 30004);
    // 接口调用: 登录
    cli->login("aubo", "123456");

    auto rtde_cli = std::make_shared<RtdeClient>();
    // 接口调用: 连接到 RTDE 服务
    rtde_cli->connect(LOCAL_IP, 30010);
    // 接口调用: 登录
    rtde_cli->login("aubo", "123456");

    int topic = rtde_cli->setTopic(false, { "R1_message" }, 200, 0);
    if (topic < 0) {
        std::cout << "Set topic fail!" << std::endl;
        return -1;
    }
    rtde_cli->subscribe(topic, [](InputParser &parser) {
        arcs::common_interface::RobotMsgVector msgs;
        msgs = parser.popRobotMsgVector();
        for (size_t i = 0; i < msgs.size(); i++) {
            auto &msg = msgs[i];
            printlog(msg.level, msg.source.c_str(), msg.code);
        }
    });

    //关节运动
    exampleMovej(cli);
    rtde_cli->removeTopic(false, topic);
    rtde_cli->logout();
    rtde_cli->disconnect();
}
