#include <math.h>
#include "aubo_sdk/rpc.h"
#include "aubo_sdk/rtde.h"
#include <aubo/error_stack/error_stack.h>
using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// 实现阻塞功能: 当机械臂运动到目标路点时，程序再往下执行
void waitArrivel(RobotInterfacePtr impl)
{
    while (impl->getMotionControl()->getExecId() == -1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
    auto id = impl->getMotionControl()->getExecId();
    while (1) {
        auto id1 = impl->getMotionControl()->getExecId();
        if (id != id1) {
            break;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

// 直线运动
void exampleMovel(RpcClientPtr impl)
{
    // 关节角，单位: 弧度
    std::vector<double> q1 = { -2.40194, 0.103747, 1.7804,
                               0.108636, 1.57129,  -2.6379 };

    std::vector<double> p1 = { 0.764244, -0.0291056, 0.439062,
                               3.05162,  0.032431,   1.80416 };
    // 位姿
    std::vector<double> Waypoint_0_p = { -0.155944, -0.727344, 0.439066,
                                         3.05165,   0.0324355, 1.80417 };
    std::vector<double> Waypoint_0_q = { -1.63013,  -0.727936, 0.558376,
                                         -0.279454, 1.66626,   -1.86472 };
    // 位姿
    std::vector<double> Waypoint_1_p = { -0.581143, -0.357548, 0.439066,
                                         3.05165,   0.0324355, 1.80417 };
    std::vector<double> Waypoint_1_q = { -2.42089,  -0.529301, 0.867566,
                                         -0.238376, 1.64145,   -2.658 };
    // 位姿
    std::vector<double> Waypoint_2_p = { 0.503502, -0.420646, 0.439066,
                                         3.05165,  0.0324355, 1.80417 };
    std::vector<double> Waypoint_2_q = { -0.515177, -0.402876, 1.09792,
                                         0.0179699, 1.60827,   -0.748361 };
    // 接口调用: 获取机器人的名字
    auto robot_name = impl->getRobotNames().front();

    // 接口调用: 关节运动
    impl->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0);
    // 实现阻塞
    waitArrivel(impl->getRobotInterface(robot_name));

    int ret = 0;                       // 函数返回值
    int cnt = 0;                       // 计数变量
    std::vector<double> cur;           // 当前的关节角
    std::vector<double> inverse_value; // 逆解得到的关节角

    for (;;) {
        printf("goto p1\n");
        // 接口调用: 直线运动
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_0_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);
        // 循环直至获取逆解 Waypoint_0_p 的关节角
        // 如果逆解失败，则程序会停止执行
        while (1) {
            cur.clear();
            // 接口调用: 获取当前的关节角
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            // 接口调用: 获得 Waypoint_0_p 的逆解结果
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_0_p);
            // 当 inverseKinematics 函数的返回值 >= 0 时，跳出循环
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            // 当 cnt++ > 10 时，程序停止执行
            if (cnt++ > 10) {
                exit(-1);
            }

            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        // 阻塞
        waitArrivel(impl->getRobotInterface(robot_name));
        inverse_value.clear();

        printf("goto p2\n");
        // 接口调用: 直线运动
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_1_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);
        // 循环直至获取逆解 Waypoint_1_p 的关节角
        // 如果逆解失败，则程序会停止执行
        while (1) {
            cur.clear();
            // 接口调用: 获取当前的关节角
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            // 接口调用: 获得 Waypoint_1_p 的逆解结果
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_1_p);
            // 当 inverseKinematics 函数的返回值 >= 0 时，跳出循环
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            // 当 cnt++ > 10 时，程序停止执行
            if (cnt++ > 10) {
                exit(-1);
            }
            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        // 阻塞
        waitArrivel(impl->getRobotInterface(robot_name));
        inverse_value.clear();

        printf("goto p3\n");
        // 接口调用: 直线运动
        impl->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(Waypoint_2_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0,
                       0);
        // 循环直至获取逆解 Waypoint_2_p 的关节角
        // 如果逆解失败，则程序会停止执行
        while (1) {
            cur.clear();
            // 接口调用: 获取当前的关节角
            cur = impl->getRobotInterface(robot_name)
                      ->getRobotState()
                      ->getJointPositions();
            // 接口调用: 获得 Waypoint_2_p 的逆解结果
            std::tuple<std::vector<double>, int> inverse_result =
                impl->getRobotInterface(robot_name)
                    ->getRobotAlgorithm()
                    ->inverseKinematics(cur, Waypoint_2_p);
            // 当 inverseKinematics 函数的返回值 >= 0 时，跳出循环
            if (std::get<1>(inverse_result) >= 0) {
                inverse_value = std::get<0>(inverse_result);
                cnt = 0;
                break;
            }
            // 当 cnt++ > 10 时，程序停止执行
            if (cnt++ > 10) {
                exit(-1);
            }
            printf("ret:%d\n", std::get<1>(inverse_result));
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        // 阻塞
        waitArrivel(impl->getRobotInterface(robot_name));
        inverse_value.clear();
    }
}

void printlog(int level, const char *source, int code)
{
    static const char *level_names[] = { "Critical", "Error", "Warning",
                                         "Info",     "Debug", "BackTrace" };
    fprintf(stderr, "[%s] %s - %d %s\n", level_names[level], source, code,
            arcs::error_stack::errorCode2Str(code));
}

/**
 * 功能: 机械臂直线运动
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 直线运动: 先关节运动起始位置，然后循环依次以直线运动的方式经过3个路点
 */
#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(LOCAL_IP, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");

    auto rtde_cli = std::make_shared<RtdeClient>();
    // 接口调用: 连接到 RTDE 服务
    rtde_cli->connect(LOCAL_IP, 30010);
    // 接口调用: 登录
    rtde_cli->login("aubo", "123456");

    int topic = rtde_cli->setTopic(false, { "R1_message" }, 200, 0);
    if (topic < 0) {
        std::cout << "Set topic fail!" << std::endl;
        return -1;
    }

    rtde_cli->subscribe(topic, [](InputParser &parser) {
        arcs::common_interface::RobotMsgVector msgs;
        msgs = parser.popRobotMsgVector();
        for (size_t i = 0; i < msgs.size(); i++) {
            auto &msg = msgs[i];
            printlog(msg.level, msg.source.c_str(), msg.code);
        }
    });
    // 直线运动
    exampleMovel(rpc_cli);

    rtde_cli->removeTopic(false, topic);
    rtde_cli->logout();
    rtde_cli->disconnect();
}
