#include "aubo_sdk/rpc.h"

using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

// 机械臂上电
void exampleStartup(RpcClientPtr cli)
{
    // 接口调用: 获取机器人的名字
    auto robot_name = cli->getRobotNames().front();
    // 当机器人不处于运行模式时，执行程序
    if (RobotModeType::Running != cli->getRobotInterface(robot_name)
                                      ->getRobotState()
                                      ->getRobotModeType()) {
        // 接口调用: 发起机器人上电请求
        cli->getRobotInterface(robot_name)->getRobotManage()->poweron();
        // 循环直至机械臂上电成功
        while (1) {
            if (RobotModeType::Idle == cli->getRobotInterface(robot_name)
                                           ->getRobotState()
                                           ->getRobotModeType()) {
                printf("上电成功:%d\n", cli->getRobotInterface(robot_name)
                                            ->getRobotState()
                                            ->getRobotModeType());
                break;
            }
            printf("正在上电:%d\n", cli->getRobotInterface(robot_name)
                                        ->getRobotState()
                                        ->getRobotModeType());
            std::this_thread::sleep_for(std::chrono::seconds(1));
        }

        // 接口调用: 发起机器人启动请求
        cli->getRobotInterface(robot_name)->getRobotManage()->startup();
        // 循环直至机械臂松刹车成功
        while (1) {
            if (RobotModeType::Running == cli->getRobotInterface(robot_name)
                                              ->getRobotState()
                                              ->getRobotModeType()) {
                printf("松刹车成功:%d\n", cli->getRobotInterface(robot_name)
                                              ->getRobotState()
                                              ->getRobotModeType());
                break;
            }
            printf("正在松刹车:%d\n", cli->getRobotInterface(robot_name)
                                          ->getRobotState()
                                          ->getRobotModeType());
            std::this_thread::sleep_for(std::chrono::seconds(1));
        }
    }
}

// 机械臂断电
void examplePoweroff(RpcClientPtr cli)
{
    // 接口调用: 获取机器人的名字
    auto robot_name = cli->getRobotNames().front();
    // 接口调用: 机械臂断电
    cli->getRobotInterface(robot_name)->getRobotManage()->poweroff();
}

/**
 * 功能: 机械臂上电与断电
 * 步骤:
 * 第一步: 连接到 RPC 服务、登录
 * 第二步: 机械臂上电
 * 第三步: 机械臂断电
 */
#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(LOCAL_IP, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");
    // 机械臂上电
    exampleStartup(rpc_cli);
    // 机械臂断电
    //    examplePoweroff(rpc_cli);
}
