#include "aubo_sdk/rpc.h"

using namespace arcs::aubo_sdk;

// 模板:打印类型为std::vector<T>的变量
template <typename T>
void printVec(std::vector<T> param, std::string name)
{
    for (int i = 0; i < param.size(); i++) {
        std::cout << param.at(i);
        if (i != param.size() - 1) {
            std::cout << ", ";
        }
    }
    std::cout << std::endl;
}

// 获取机械臂状态
void exampleState(RpcClientPtr cli)
{
    auto robot_name = cli->getRobotNames().front();

    // 接口调用:获取机器人的模式状态
    RobotModeType robot_mode_type;
    robot_mode_type =
        cli->getRobotInterface(robot_name)->getRobotState()->getRobotModeType();
    std::cout << "机器人的模式状态:" << robot_mode_type << std::endl;

    // 接口调用:获取安全模式
    SafetyModeType safety_mode_type;
    safety_mode_type = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getSafetyModeType();
    std::cout << "安全模式:" << safety_mode_type << std::endl;

    // 接口调用:机器人是否已经停止下来
    bool is_steady;
    is_steady = cli->getRobotInterface(robot_name)->getRobotState()->isSteady();
    std::cout << "机器人是否已经停止下来:" << is_steady << std::endl;

    // 接口调用:机器人是否已经在安全限制之内
    bool is_within_safety_limits;
    is_within_safety_limits = cli->getRobotInterface(robot_name)
                                  ->getRobotState()
                                  ->isWithinSafetyLimits();
    std::cout << "机器人是否已经在安全限制之内:" << is_within_safety_limits
              << std::endl;

    // 接口调用:获取TCP的位姿
    std::vector<double> tcp_pose;
    tcp_pose =
        cli->getRobotInterface(robot_name)->getRobotState()->getTcpPose();
    std::cout << "TCP的位姿：";
    printVec<double>(tcp_pose, "tcp_pose");

    // 接口调用:获取当前目标位姿
    std::vector<double> target_tcp_pose;
    target_tcp_pose =
        cli->getRobotInterface(robot_name)->getRobotState()->getTargetTcpPose();
    std::cout << "当前目标位姿：";
    printVec<double>(target_tcp_pose, "target_tcp_pose");

    // 接口调用:获取工具端的位姿（不带TCP偏移）
    std::vector<double> tool_pose;
    tool_pose =
        cli->getRobotInterface(robot_name)->getRobotState()->getToolPose();
    std::cout << "工具端的位姿（不带TCP偏移）：";
    printVec<double>(tool_pose, "tool_pose");

    // 接口调用:获取TCP速度
    std::vector<double> tcp_speed;
    tcp_speed =
        cli->getRobotInterface(robot_name)->getRobotState()->getTcpSpeed();
    std::cout << "TCP速度：";
    printVec<double>(tcp_speed, "tcp_speed");

    //接口调用:获取TCP的力/力矩
    std::vector<double> tcp_force;
    tcp_force =
        cli->getRobotInterface(robot_name)->getRobotState()->getTcpForce();
    std::cout << "TCP的力/力矩：";
    printVec<double>(tcp_force, "tcp_force");

    // 接口调用:获取肘部的位置
    std::vector<double> elbow_postion;
    elbow_postion = cli->getRobotInterface(robot_name)
                        ->getRobotState()
                        ->getElbowPosistion();
    std::cout << "肘部的位置：";
    printVec<double>(elbow_postion, "elbow_postion");

    // 接口调用:获取肘部速度
    std::vector<double> elbow_velocity;
    elbow_velocity =
        cli->getRobotInterface(robot_name)->getRobotState()->getElbowVelocity();
    std::cout << "肘部速度：";
    printVec<double>(elbow_velocity, "elbow_velocity");

    // 接口调用:获取基座力/力矩
    std::vector<double> base_force;
    base_force =
        cli->getRobotInterface(robot_name)->getRobotState()->getBaseForce();
    std::cout << "基座力/力矩：";
    printVec<double>(base_force, "base_force");

    // 接口调用:获取TCP目标位姿
    std::vector<double> tcp_target_pose;
    tcp_target_pose =
        cli->getRobotInterface(robot_name)->getRobotState()->getTcpTargetPose();
    std::cout << "TCP目标位姿：";
    printVec<double>(tcp_target_pose, "tcp_target_pose");

    // 接口调用:获取TCP目标速度
    std::vector<double> tcp_target_speed;
    tcp_target_speed = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getTcpTargetSpeed();
    std::cout << "TCP目标速度：";
    printVec<double>(tcp_target_speed, "tcp_target_speed");

    // 接口调用:获取TCP目标力/力矩
    std::vector<double> tcp_target_force;
    tcp_target_force = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getTcpTargetForce();
    std::cout << "TCP目标力/力矩：";
    printVec<double>(tcp_target_force, "tcp_target_force");

    // 接口调用:获取机械臂关节标志
    std::vector<JointStateType> joint_state;
    joint_state =
        cli->getRobotInterface(robot_name)->getRobotState()->getJointState();
    std::cout << "机械臂关节标志：";
    printVec<JointStateType>(joint_state, "joint_state");

    // 接口调用:获取关节的伺服状态
    std::vector<JointServoModeType> joint_servo_mode;
    joint_servo_mode = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getJointServoMode();
    std::cout << "关节的伺服状态：";
    printVec<JointServoModeType>(joint_servo_mode, "joint_servo_mode");

    // 接口调用:获取机械臂关节角度
    std::vector<double> joint_positions;
    joint_positions = cli->getRobotInterface(robot_name)
                          ->getRobotState()
                          ->getJointPositions();
    std::cout << "机械臂关节角度：";
    printVec<double>(joint_positions, "joint_positions");

    // 获取机械臂关节速度
    std::vector<double> joint_speeds;
    joint_speeds =
        cli->getRobotInterface(robot_name)->getRobotState()->getJointSpeeds();
    std::cout << "机械臂关节速度：";
    printVec<double>(joint_speeds, "joint_speeds");

    // 接口调用:获取机械臂关节加速度
    std::vector<double> joint_acc;
    joint_acc = cli->getRobotInterface(robot_name)
                    ->getRobotState()
                    ->getJointAccelerations();
    std::cout << "机械臂关节加速度：";
    printVec<double>(joint_acc, "joint_acc");

    // 接口调用:获取机械臂关节力矩
    std::vector<double> joint_torque_sensors;
    joint_torque_sensors = cli->getRobotInterface(robot_name)
                               ->getRobotState()
                               ->getJointTorqueSensors();
    std::cout << "机械臂关节力矩：";
    printVec<double>(joint_torque_sensors, "joint_torque_sensors");

    // 接口调用:获取底座力传感器读数
    std::vector<double> base_force_sensors;
    base_force_sensors = cli->getRobotInterface(robot_name)
                             ->getRobotState()
                             ->getBaseForceSensor();
    std::cout << "底座力传感器读数：";
    printVec<double>(base_force_sensors, "base_force_sensors");

    // 接口调用:获取TCP力传感器读数
    std::vector<double> tcp_force_sensors;
    tcp_force_sensors = cli->getRobotInterface(robot_name)
                            ->getRobotState()
                            ->getTcpForceSensors();
    std::cout << "TCP力传感器读数：";
    printVec<double>(tcp_force_sensors, "tcp_force_sensors");

    // 接口调用:获取机械臂关节电流
    std::vector<double> joint_currents;
    joint_currents =
        cli->getRobotInterface(robot_name)->getRobotState()->getJointCurrents();
    std::cout << "机械臂关节电流：";
    printVec<double>(joint_currents, "joint_currents");

    // 接口调用:获取机械臂关节电压
    std::vector<double> joint_voltages;
    joint_voltages =
        cli->getRobotInterface(robot_name)->getRobotState()->getJointVoltages();
    std::cout << "机械臂关节电压：";
    printVec<double>(joint_voltages, "joint_voltages");

    // 接口调用:获取机械臂关节温度
    std::vector<double> joint_temperatures;
    joint_temperatures = cli->getRobotInterface(robot_name)
                             ->getRobotState()
                             ->getJointTemperatures();
    std::cout << "机械臂关节温度：";
    printVec<double>(joint_temperatures, "joint_temperatures");

    // 接口调用:获取关节 UniqueId
    std::vector<std::string> joint_unique_ids;
    joint_unique_ids = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getJointUniqueIds();
    std::cout << "关节 UniqueId：";
    printVec<std::string>(joint_unique_ids, "joint_unique_ids");

    // 接口调用:获取关节固件版本
    std::vector<int> joint_firmware_versions;
    joint_firmware_versions = cli->getRobotInterface(robot_name)
                                  ->getRobotState()
                                  ->getJointFirmwareVersions();
    std::cout << "关节固件版本：";
    printVec<int>(joint_firmware_versions, "joint_firmware_versions");

    // 接口调用:获取关节硬件版本
    std::vector<int> joint_hardward_versions;
    joint_hardward_versions = cli->getRobotInterface(robot_name)
                                  ->getRobotState()
                                  ->getJointHardwareVersions();
    std::cout << "关节硬件版本：";
    printVec<int>(joint_hardward_versions, "joint_hardward_versions");

    // 接口调用:获取主板 UniqueId
    std::string master_board_unique_id;
    master_board_unique_id = cli->getRobotInterface(robot_name)
                                 ->getRobotState()
                                 ->getMasterBoardUniqueId();
    std::cout << "主板 UniqueId：" << master_board_unique_id << std::endl;

    // 接口调用:获取MasterBoard固件版本
    int master_board_firmware_version;
    master_board_firmware_version = cli->getRobotInterface(robot_name)
                                        ->getRobotState()
                                        ->getMasterBoardFirmwareVersion();
    std::cout << "MasterBoard固件版本：" << master_board_firmware_version
              << std::endl;

    // 接口调用:获取MasterBoard硬件版本
    int master_board_hardware_version;
    master_board_hardware_version = cli->getRobotInterface(robot_name)
                                        ->getRobotState()
                                        ->getMasterBoardHardwareVersion();
    std::cout << "MasterBoard硬件版本：" << master_board_hardware_version
              << std::endl;

    // 接口调用:获取从板 UniqueId
    std::string slave_board_unique_id;
    slave_board_unique_id = cli->getRobotInterface(robot_name)
                                ->getRobotState()
                                ->getSlaveBoardUniqueId();
    std::cout << "从板 UniqueId：" << slave_board_unique_id << std::endl;

    // 接口调用:获取SlaveBoard固件版本
    int slave_board_firmware_version;
    slave_board_firmware_version = cli->getRobotInterface(robot_name)
                                       ->getRobotState()
                                       ->getSlaveBoardFirmwareVersion();
    std::cout << "SlaveBoard固件版本：" << slave_board_firmware_version
              << std::endl;

    // 接口调用:获取SlaveBoard硬件版本
    int slave_board_hardware_version;
    slave_board_hardware_version = cli->getRobotInterface(robot_name)
                                       ->getRobotState()
                                       ->getSlaveBoardHardwareVersion();
    std::cout << "SlaveBoard硬件版本：" << slave_board_hardware_version
              << std::endl;

    // 接口调用:获取工具端全球唯一ID
    std::string tool_unique_id;
    tool_unique_id =
        cli->getRobotInterface(robot_name)->getRobotState()->getToolUniqueId();
    std::cout << "工具端全球唯一ID：" << tool_unique_id << std::endl;

    // 接口调用:获取工具端固件版本
    int tool_firmware_version;
    tool_firmware_version = cli->getRobotInterface(robot_name)
                                ->getRobotState()
                                ->getToolFirmwareVersion();
    std::cout << "工具端固件版本：" << tool_firmware_version << std::endl;

    // 接口调用:获取工具端硬件版本
    int tool_hardware_version;
    tool_hardware_version = cli->getRobotInterface(robot_name)
                                ->getRobotState()
                                ->getToolHardwareVersion();
    std::cout << "工具端硬件版本：" << tool_hardware_version << std::endl;

    // 接口调用:获取底座全球唯一ID
    std::string pedestal_unique_id;
    pedestal_unique_id = cli->getRobotInterface(robot_name)
                             ->getRobotState()
                             ->getPedestalUniqueId();
    std::cout << "底座全球唯一ID：" << pedestal_unique_id << std::endl;

    // 接口调用:获取底座固件版本
    int pedestal_firmware_version;
    pedestal_firmware_version = cli->getRobotInterface(robot_name)
                                    ->getRobotState()
                                    ->getPedestalFirmwareVersion();
    std::cout << "底座固件版本：" << pedestal_firmware_version << std::endl;

    // 接口调用:获取底座硬件版本
    int pedestal_hardware_version;
    pedestal_hardware_version = cli->getRobotInterface(robot_name)
                                    ->getRobotState()
                                    ->getPedestalHardwareVersion();
    std::cout << "底座硬件版本：" << pedestal_hardware_version << std::endl;

    // 接口调用:获取机械臂关节目标位置角度
    std::vector<double> joint_target_positions;
    joint_target_positions = cli->getRobotInterface(robot_name)
                                 ->getRobotState()
                                 ->getJointTargetPositions();
    std::cout << "关节目标位置角度：";
    printVec<double>(joint_target_positions, "joint_target_positions");

    // 接口调用:获取机械臂关节目标速度
    std::vector<double> joint_target_speeds;
    joint_target_speeds = cli->getRobotInterface(robot_name)
                              ->getRobotState()
                              ->getJointTargetSpeeds();
    std::cout << "机械臂关节目标速度：";
    printVec<double>(joint_target_speeds, "joint_target_speeds");

    // 接口调用:获取机械臂关节目标加速度
    std::vector<double> joint_target_acc;
    joint_target_acc = cli->getRobotInterface(robot_name)
                           ->getRobotState()
                           ->getJointTargetAccelerations();
    std::cout << "机械臂关节目标加速度：";
    printVec<double>(joint_target_acc, "joint_target_acc");

    // 接口调用:获取机械臂关节目标力矩
    std::vector<double> joint_target_torques;
    joint_target_torques = cli->getRobotInterface(robot_name)
                               ->getRobotState()
                               ->getJointTargetTorques();
    std::cout << "机械臂关节目标力矩：";
    printVec<double>(joint_target_torques, "joint_target_torques");

    // 接口调用:获取机械臂关节目标电流
    std::vector<double> joint_target_currents;
    joint_target_currents = cli->getRobotInterface(robot_name)
                                ->getRobotState()
                                ->getJointTargetCurrents();
    std::cout << "机械臂关节目标电流：";
    printVec<double>(joint_target_currents, "joint_target_currents");

    // 接口调用:获取控制柜温度
    double control_box_temperature;
    control_box_temperature = cli->getRobotInterface(robot_name)
                                  ->getRobotState()
                                  ->getControlBoxTemperature();
    std::cout << "控制柜温度：" << control_box_temperature << std::endl;

    // 接口调用:获取母线电压
    double main_voltage;
    main_voltage =
        cli->getRobotInterface(robot_name)->getRobotState()->getMainVoltage();
    std::cout << "母线电压：" << main_voltage << std::endl;

    // 接口调用:获取母线电流
    double main_current;
    main_current =
        cli->getRobotInterface(robot_name)->getRobotState()->getMainCurrent();
    std::cout << "母线电流：" << main_current << std::endl;

    // 接口调用:获取机器人电压
    double robot_voltage;
    robot_voltage =
        cli->getRobotInterface(robot_name)->getRobotState()->getRobotVoltage();
    std::cout << "机器人电压：" << robot_voltage << std::endl;

    // 接口调用:获取机器人电流
    double robot_current;
    robot_current =
        cli->getRobotInterface(robot_name)->getRobotState()->getRobotCurrent();
    std::cout << "机器人电流：" << robot_current << std::endl;
}

#define ip_local "127.0.0.1"
/**
 * 功能:获取机械臂状态
 * 步骤:
 * 第一步:连接 RPC 服务、机械臂登录
 * 第二步:获取机械臂状态
 */
int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    rpc_cli->connect(ip_local, 30004);
    // 接口调用: 登录
    rpc_cli->login("aubo", "123456");

    // 获取机械臂状态
    exampleState(rpc_cli);
}
